@@ -3,6 +3,7 @@ import os
 WORKSPACE_NAME = os.environ.get('WORKSPACE_NAME', None)
 WORKSPACE_GOOGLE_PROJECT = os.environ.get('GOOGLE_PROJECT')  # This env var is set in Terra notebooks
 TERRA_DEPLOYMENT_ENV = os.environ.get('TERRA_DEPLOYMENT_ENV', 'prod')
+MARTHA_URL_VERSION = os.environ.get('MARTHA_URL_VERSION', 'martha_v3')
 
 if not WORKSPACE_GOOGLE_PROJECT:
     WORKSPACE_GOOGLE_PROJECT = os.environ.get('GCP_PROJECT')  # Useful for running outside of notebook
@@ -15,4 +16,9 @@ _GS_SCHEMA = 'gs://'
 if WORKSPACE_BUCKET is not None and WORKSPACE_BUCKET.startswith(_GS_SCHEMA):
     WORKSPACE_BUCKET = WORKSPACE_BUCKET[len(_GS_SCHEMA):]  # Chop off the bucket schema
 
+MULTIPART_THRESHOLD = 1024 * 1024 * 32
+IO_CONCURRENCY = 3
+
+MARTHA_URL = f"https://us-central1-broad-dsde-{TERRA_DEPLOYMENT_ENV}.cloudfunctions.net/{MARTHA_URL_VERSION}"
+
 from terra_notebook_utils import drs, profile, table, vcf, workspace
