@@ -3,15 +3,17 @@ import os
 import logging
 import warnings
 from math import ceil
+from concurrent.futures import ThreadPoolExecutor
 from contextlib import closing
-from concurrent.futures import as_completed
 
 from google.cloud.storage import Client
 from google.oauth2 import service_account
 import google.auth
 import gs_chunked_io as gscio
+from gs_chunked_io.async_collections import AsyncSet
+import google.cloud.storage.bucket as GSBucket
 
-from terra_notebook_utils import WORKSPACE_BUCKET, TERRA_DEPLOYMENT_ENV
+from terra_notebook_utils import WORKSPACE_BUCKET, TERRA_DEPLOYMENT_ENV, MULTIPART_THRESHOLD, IO_CONCURRENCY
 from terra_notebook_utils.progress import ProgressBar
 
 
@@ -19,7 +21,6 @@ logging.getLogger("google.resumable_media.requests.download").setLevel(logging.W
 logging.getLogger("gs_chunked_io.writer").setLevel(logging.WARNING)
 
 logger = logging.getLogger(__name__)
-logger.setLevel(logging.WARNING)
 
 
 # Suppress the annoying google gcloud _CLOUD_SDK_CREDENTIALS_WARNING warnings
@@ -68,43 +69,45 @@ def get_client(credentials_data: dict=None, project: str=None):
         client._credentials.refresh(google.auth.transport.requests.Request())
     return client
 
-def oneshot_copy(src_bucket, dst_bucket, src_key, dst_key):
+def oneshot_copy(src_bucket: GSBucket, dst_bucket: GSBucket, src_key: str, dst_key: str):
     """
     Download an object into memory from `src_bucket` and upload it to `dst_bucket`
     """
+    logger.info(f"Copying gs://{src_bucket.name}/{src_key} to gs://{dst_bucket.name}/{dst_key}")
     fh = io.BytesIO()
     src_bucket.blob(src_key).download_to_file(fh)
     fh.seek(0)
     dst_bucket.blob(dst_key).upload_from_file(fh)
+    assert src_bucket.get_blob(src_key).crc32c == dst_bucket.get_blob(dst_key).crc32c
 
-def multipart_copy(src_bucket, dst_bucket, src_key, dst_key):
+def multipart_copy(src_bucket: GSBucket, dst_bucket: GSBucket, src_key: str, dst_key: str):
     """
     Download/upload an object in parts from `src_bucket` to `dst_bucket`
     """
     src_blob = src_bucket.get_blob(src_key)
-    print(f"Copying from {src_bucket.name}/{src_key}")
-    print(f"Copying to {dst_bucket.name}/{dst_key}")
+    print(f"Copying gs://{src_bucket.name}/{src_key} to gs://{dst_bucket.name}/{dst_key}")
     number_of_chunks = ceil(src_blob.size / gscio.reader.default_chunk_size)
     progress_bar = ProgressBar(1 + number_of_chunks,
                                prefix="Copying:",
                                size=src_blob.size // 1024 ** 2,
                                units="MB")
-    with closing(progress_bar):
-        with gscio.AsyncWriter(dst_key, dst_bucket) as writer:
-            for chunk_number, chunk in gscio.AsyncReader.for_each_chunk_async(src_blob):
-                writer.put_part_async(chunk_number, chunk)
-                progress_bar.update()
-        progress_bar.update()
+    with ThreadPoolExecutor(max_workers=2 * IO_CONCURRENCY) as e:
+        async_uploads = AsyncSet(e, IO_CONCURRENCY)
+        async_downloads = AsyncSet(e, IO_CONCURRENCY)
+        with closing(progress_bar):
+            with gscio.AsyncPartUploader(dst_key, dst_bucket, async_uploads) as writer:
+                for chunk_number, chunk in gscio.for_each_chunk_async(src_blob, async_downloads):
+                    writer.put_part(chunk_number, chunk)
+                    progress_bar.update()
+            progress_bar.update()
+        assert src_bucket.get_blob(src_key).crc32c == dst_bucket.get_blob(dst_key).crc32c
 
-def copy(src_bucket, dst_bucket, src_key, dst_key, multipart_threshold=1024 * 1024 * 32):
+def copy(src_bucket: GSBucket, dst_bucket: GSBucket, src_key: str, dst_key: str):
     src_blob = src_bucket.get_blob(src_key)
-    if multipart_threshold >= src_blob.size:
+    if MULTIPART_THRESHOLD >= src_blob.size:
         oneshot_copy(src_bucket, dst_bucket, src_key, dst_key)
     else:
         multipart_copy(src_bucket, dst_bucket, src_key, dst_key)
-    src_blob.reload()
-    dst_blob = dst_bucket.get_blob(dst_key)
-    assert src_blob.crc32c == dst_blob.crc32c
 
 def list_bucket(prefix="", bucket=WORKSPACE_BUCKET):
     for blob in get_client().bucket(bucket).list_blobs(prefix=prefix):
